/* -*- mode: c++; tab-width: 4; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <qapplication.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qtabbar.h>
#include <qevent.h>

#include "configurationdialog.h"
#include "configuration.h"
#include "configview.h"
#include "helpers.h"
#include "addonmanager.h"
#include "config/generalconfig.h"
#include "config/mplayerconfig.h"
#include "formatsconfig.h"
#include "hotkeyconfig.h"
#include "config/hotareaconfig.h"
#include "config/uiconfig.h"
#include "config/coverartconfig.h"
#include "compathack.h"

#ifdef QT4
ConfigurationDialog::ConfigurationDialog(const Configuration &config, QWidget *parent)
	:	QTabbedDialog(parent),
		configViews(),
		workingConfiguration_(config)
{
#ifdef MAEMO
	setAttribute(Qt::WA_Maemo5StackedWindow);
	setWindowFlags(Qt::Window);
	setModal(true);
#else
	setModal(true);
#endif

	createWidgets();
}
#else
ConfigurationDialog::ConfigurationDialog(const Configuration &config, QWidget *parent)
	:	QTabbedDialog(parent, 0, true),
		configViews(),
		workingConfiguration_(config)
{
	createWidgets();
}
#endif

ConfigurationDialog::~ConfigurationDialog()
{
}

void ConfigurationDialog::createWidgets()
{
	setWindowTitle(tr("Preferences"));
#ifdef QTOPIA
	setWState(WState_Reserved1);
#endif

#ifdef QT4
	tabWidget_ = new QTabWidget(this);
#endif

	generalConfig = new GeneralConfig(workingConfiguration_, this);
	connect(generalConfig, SIGNAL(defaultButtonClicked()), this, SLOT(setDefaultSetting()));
	connect(generalConfig, SIGNAL(nukeMediaCacheButtonClicked()), this, SIGNAL(nukeMediaCacheButtonClicked()));
	initializeAndAddConfigViewTab(generalConfig, generalConfig, tr("General"));

	mplayerConfig = new MPlayerConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(mplayerConfig, mplayerConfig, tr("MPlayer"));

	formatsConfig = new FormatsConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(formatsConfig, formatsConfig, tr("Formats"));

	hotKeyConfig = new HotKeyConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(hotKeyConfig, hotKeyConfig, tr("Hot Keys"));

	hotAreaConfig = new HotAreaConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(hotAreaConfig, hotAreaConfig, tr("Hot Areas"));

	uiConfig = new UIConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(uiConfig, uiConfig, tr("User Interface"));

	coverArtConfig = new CoverArtConfig(workingConfiguration_, this);
	initializeAndAddConfigViewTab(coverArtConfig, coverArtConfig, tr("Cover Art"));

	qAddOnManager.hookConfigurationDialog(this);

	setStatesByConfiguration(workingConfiguration_);

#ifdef QT4

	QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget_);

#ifndef MAEMO
	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
	connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	mainLayout->addWidget(buttonBox);
#endif

    setLayout(mainLayout);
#endif

	// Hide buttons
#ifdef QTOPIA
	setOkButton(QString::null);
#endif
}

void ConfigurationDialog::initializeAndAddConfigViewTab(BaseConfigView *cv, QWidget *view, const QString &tabCaption)
{
	cv->createWidgets();
#ifdef QT4
	tabWidget_->addTab(view, tabCaption);
#else
	addTab(view, tabCaption);
#endif
	configViews.append(cv);
}

void ConfigurationDialog::showEvent(QShowEvent *e)
{
#ifdef QTOPIA
	showMaximized();
#endif;
	QDialog::showEvent(e);
}

void ConfigurationDialog::hideEvent(QHideEvent *e)
{
	QDialog::hideEvent(e);
}

void ConfigurationDialog::setDefaultSetting()
{
	workingConfiguration_ = Configuration();
	setStatesByConfiguration(workingConfiguration_);
}

void ConfigurationDialog::setStatesByConfiguration(const Configuration& config)
{
	for (int i = 0; i < configViews.count(); ++i)
		configViews.at(i)->setStatesByConfiguration(config);

	emit settingStatesByConfiguration(config);
}

void ConfigurationDialog::saveStatesToConfiguration()
{
	for (int i = 0; i < configViews.count(); ++i)
		configViews.at(i)->saveStatesToConfiguration();

	emit savingStatesToConfiguration();
}

void ConfigurationDialog::showMPlayerConfig()
{
#ifdef QT4
	tabWidget_->setCurrentWidget(mplayerConfig);
#else
	showPage(mplayerConfig);
#endif
	mplayerConfig->setFocus();
	mplayerConfig->focusMPlayerPathWidget();
}

int ConfigurationDialog::exec()
{
	loadWidgetPosition(this, qGlobalConfig.configFilename(), "ConfigDialog");
#ifdef QT4
	int result = QDialog::exec();
#ifdef MAEMO
	result = QDialog::Accepted;
#endif
#else
	int result = QTabDialog::exec();
#endif
	saveStatesToConfiguration();
	saveWidgetPosition(this, qGlobalConfig.configFilename(), "ConfigDialog");
	return result;
}
